@@ -8,7 +8,7 @@
 # @option      -x --opt7[x|y|z]    A option with choices
 # @option      -y --opt8[=x|y|z]   A option with choices and default value
 # @option      -z --opt9![x|y|z]   A required option with choices
-# @option         --opt10[`_fn2`]  A option with choices from fn 
+# @option      -w  --opt10[`_fn2`]  A option with choices from fn 
 
 _fn() {
     echo abc
@@ -18,7 +18,7 @@ _fn2() {
     echo "x y z"
 }
 
-eval "$(argc "$0" "$@")"
+eval "$(argc --argc-eval "$0" "$@")"
 
 
 ( set -o posix ; set ) | grep argc_ # print variables with argc_ prefix
