@@ -1,3 +1,5 @@
+use clap::builder::StyledStr;
+
 use crate::utils::hyphens_to_underscores;
 
 pub const VARIABLE_PREFIX: &str = "argc";
@@ -12,10 +14,11 @@ pub enum ArgcValue {
     PositionalMultiple(String, Vec<String>),
     CmdFn(String),
     ParamFn(String),
+    ClapError((StyledStr, i32)),
 }
 
 impl ArgcValue {
-    pub fn to_shell(values: Vec<Self>) -> String {
+    pub fn to_shell(values: Vec<Self>, no_color: bool) -> String {
         let mut variables = vec![];
         let mut positional_args = vec![];
         for value in values {
@@ -79,7 +82,20 @@ impl ArgcValue {
                     }
                 }
                 ArgcValue::ParamFn(name) => {
-                    variables.push(format!("{name};exit;"));
+                    if positional_args.is_empty() {
+                        variables.push(format!("{name};exit;"));
+                    } else {
+                        variables.push(format!("{} {};exit;", name, positional_args.join(" ")));
+                    }
+                }
+                ArgcValue::ClapError((error, exit)) => {
+                    variables.clear();
+                    let error = if no_color {
+                        error.to_string()
+                    } else {
+                        error.ansi().to_string()
+                    };
+                    variables.push(format!("cat >&2 <<-'EOF' \n{}\nEOF\nexit {}", error, exit));
                 }
             }
         }
