@@ -6,7 +6,8 @@ pub mod parser;
 pub mod utils;
 
 use anyhow::Error;
-pub use cli::eval;
-pub use completion::compgen;
+pub use argc_value::ArgcValue;
+pub use cli::{eval, export};
+pub use completion::{compgen, Shell};
 
 pub type Result<T> = std::result::Result<T, Error>;
