@@ -7,6 +7,7 @@ use crate::utils::{
 use anyhow::{bail, Result};
 use clap::builder::PossibleValuesParser;
 use clap::{Arg, ArgAction, ArgMatches};
+use serde::Serialize;
 use std::collections::HashMap;
 use std::fmt::Write;
 
@@ -39,101 +40,112 @@ impl ParamData {
     }
 }
 
-pub trait Param {
-    fn name(&self) -> &str;
-    fn tag_name(&self) -> &str;
-    fn render(&self) -> String;
-    fn build_arg(&self, index: usize) -> Result<Arg>;
-    fn get_arg_value(&self, matches: &ArgMatches) -> Option<ArgcValue>;
-    fn detect_conflict(&self, names: &mut ParamNames, pos: Position) -> Result<()>;
-    fn is_positional(&self) -> bool;
+#[derive(Debug, PartialEq, Eq, Clone, Serialize)]
+#[serde(tag = "type")]
+pub enum Param {
+    Flag(FlagParam),
+    Option(OptionParam),
+    Positional(PositionalParam),
 }
 
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Debug, PartialEq, Eq, Clone, Serialize)]
 pub struct FlagParam {
     pub(crate) name: String,
     pub(crate) summary: String,
     pub(crate) short: Option<char>,
+    pub(crate) no_long: bool,
     pub(crate) multiple: bool,
 }
 
 impl FlagParam {
-    pub fn new(arg: ParamData, summary: &str, short: Option<char>) -> Self {
+    pub fn new(arg: ParamData, summary: &str, short: Option<char>, no_long: bool) -> Self {
         FlagParam {
             name: arg.name,
             short,
+            no_long,
             multiple: arg.multiple,
             summary: summary.to_string(),
         }
     }
-}
 
-impl Param for FlagParam {
-    fn name(&self) -> &str {
+    pub fn name(&self) -> &str {
         &self.name
     }
 
-    fn tag_name(&self) -> &str {
+    pub fn tag_name(&self) -> &str {
         "@flag"
     }
 
-    fn render(&self) -> String {
+    pub fn render(&self) -> String {
         let mut output = vec![];
-        render_short(&mut output, &self.short);
-        output.push(format!("--{}", self.name));
+        let multiple = if self.multiple { "*" } else { "" };
+        if self.no_long {
+            if let Some(ch) = self.short {
+                output.push(format!("-{}{}", ch, multiple));
+            }
+        } else {
+            if let Some(ch) = self.short {
+                output.push(format!("-{}", ch));
+            }
+            output.push(format!("--{}{}", self.name, multiple));
+        }
         render_summary(&mut output, &self.summary);
         output.join(" ")
     }
 
-    fn build_arg(&self, _index: usize) -> Result<Arg> {
+    pub fn build_arg(&self) -> Result<Arg> {
         let mut arg = new_arg(&self.name, &self.summary);
-        arg = arg.long(self.name.to_string());
-        if self.multiple {
-            arg = arg.action(ArgAction::Count);
-        } else {
-            arg = arg.action(ArgAction::SetTrue);
+        if !self.no_long {
+            arg = arg.long(self.name.to_string());
         }
         if let Some(s) = self.short {
             arg = arg.short(s);
         }
+        if self.name == "help" {
+            arg = arg.action(ArgAction::Help)
+        } else if self.name == "version" {
+            arg = arg.action(ArgAction::Version)
+        } else if self.multiple {
+            arg = arg.action(ArgAction::Count);
+        } else {
+            arg = arg.action(ArgAction::SetTrue);
+        }
         Ok(arg)
     }
 
-    fn get_arg_value(&self, matches: &ArgMatches) -> Option<ArgcValue> {
+    pub fn get_arg_value(&self, matches: &ArgMatches) -> Option<ArgcValue> {
         if self.multiple {
             let count = matches.get_count(&self.name);
-            Some(ArgcValue::Single(self.name.clone(), count.to_string()))
-        } else if matches.get_flag(&self.name) {
-            Some(ArgcValue::Single(self.name.clone(), "1".to_string()))
-        } else {
-            None
+            return Some(ArgcValue::Single(self.name.clone(), count.to_string()));
         }
+        if let Ok(Some(&true)) = matches.try_get_one::<bool>(&self.name) {
+            return Some(ArgcValue::Single(self.name.clone(), "1".into()));
+        };
+        None
     }
 
-    fn detect_conflict(&self, names: &mut ParamNames, pos: Position) -> Result<()> {
+    pub fn detect_conflict(&self, names: &mut ParamNames, pos: Position) -> Result<()> {
         let tag_name = self.tag_name();
         detect_name_conflict(&self.name, false, tag_name, names, pos)?;
         detect_short_name_conflict(&self.short, tag_name, names, pos)
     }
-
-    fn is_positional(&self) -> bool {
-        false
-    }
 }
 
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Debug, PartialEq, Eq, Clone, Serialize)]
 pub struct OptionParam {
     pub(crate) name: String,
     pub(crate) summary: String,
     pub(crate) short: Option<char>,
-    pub(crate) value_name: Option<String>,
+    pub(crate) no_long: bool,
     pub(crate) choices: Option<Vec<String>>,
     pub(crate) choices_fn: Option<String>,
     pub(crate) multiple: bool,
     pub(crate) required: bool,
     pub(crate) default: Option<String>,
     pub(crate) default_fn: Option<String>,
-    pub(crate) arg_value_name: String,
+    pub(crate) value_names: Vec<String>,
+    #[serde(skip_serializing)]
+    pub(crate) arg_value_names: Vec<String>,
 }
 
 impl OptionParam {
@@ -141,65 +153,88 @@ impl OptionParam {
         arg: ParamData,
         summary: &str,
         short: Option<char>,
-        value_name: Option<&str>,
+        no_long: bool,
+        value_names: &[&str],
     ) -> Self {
+        let value_names: Vec<String> = value_names.iter().map(|v| v.to_string()).collect();
+        let arg_value_names = if value_names.is_empty() {
+            vec![to_cobol_case(&arg.name)]
+        } else {
+            value_names.iter().map(|v| to_cobol_case(v)).collect()
+        };
         OptionParam {
             name: arg.name.clone(),
             summary: summary.to_string(),
+            short,
+            no_long,
             choices: arg.choices,
             choices_fn: arg.choices_fn,
             multiple: arg.multiple,
             required: arg.required,
             default: arg.default,
             default_fn: arg.default_fn,
-            short,
-            value_name: value_name.map(|v| v.to_string()),
-            arg_value_name: value_name
-                .or(Some(&arg.name))
-                .map(to_cobol_case)
-                .unwrap_or_default(),
+            value_names,
+            arg_value_names,
         }
     }
-}
 
-impl Param for OptionParam {
-    fn name(&self) -> &str {
+    pub fn name(&self) -> &str {
         &self.name
     }
 
-    fn tag_name(&self) -> &str {
+    pub fn tag_name(&self) -> &str {
         "@option"
     }
 
-    fn render(&self) -> String {
+    pub fn render(&self) -> String {
         let mut output = vec![];
-        render_short(&mut output, &self.short);
-        let name = render_name(
-            &self.name,
-            &self.choices,
-            &self.choices_fn,
-            self.multiple,
-            self.required,
-            &self.default,
-            &self.default_fn,
-        );
-        output.push(format!("--{}", name));
-        if let Some(value_name) = self.value_name.as_ref() {
+        if self.no_long {
+            let name = render_name(
+                &self.name,
+                &self.choices,
+                &self.choices_fn,
+                self.multiple,
+                self.required,
+                &self.default,
+                &self.default_fn,
+            );
+            output.push(format!("-{}", name));
+        } else {
+            if let Some(ch) = self.short {
+                output.push(format!("-{}", ch));
+            };
+            let name = render_name(
+                &self.name,
+                &self.choices,
+                &self.choices_fn,
+                self.multiple,
+                self.required,
+                &self.default,
+                &self.default_fn,
+            );
+            output.push(format!("--{}", name));
+        }
+        for value_name in &self.value_names {
             output.push(format!("<{}>", value_name));
         }
         render_summary(&mut output, &self.summary);
         output.join(" ")
     }
 
-    fn build_arg(&self, _index: usize) -> Result<Arg> {
+    pub fn build_arg(&self) -> Result<Arg> {
         let mut arg = new_arg(&self.name, &self.summary);
-        arg = arg
-            .long(self.name.to_string())
-            .required(self.required)
-            .value_name(&self.arg_value_name);
+        if !self.no_long {
+            arg = arg.long(self.name.to_string())
+        }
         if let Some(s) = self.short {
             arg = arg.short(s);
         }
+        arg = arg.required(self.required);
+        if self.arg_value_names.len() == 1 {
+            arg = arg.value_name(&self.arg_value_names[0]);
+        } else {
+            arg = arg.value_names(&self.arg_value_names);
+        }
         if self.multiple {
             let num = usize::from(self.required);
             arg = arg
@@ -218,7 +253,24 @@ impl Param for OptionParam {
         Ok(arg)
     }
 
-    fn get_arg_value(&self, matches: &ArgMatches) -> Option<ArgcValue> {
+    pub fn build_arg_loose(&self) -> Result<Arg> {
+        let mut arg = new_arg(&self.name, &self.summary);
+        if !self.no_long {
+            arg = arg.long(self.name.to_string())
+        }
+        if let Some(s) = self.short {
+            arg = arg.short(s);
+        }
+        if self.multiple {
+            arg = arg.value_delimiter(',').action(ArgAction::Append)
+        }
+        if let Some(default) = self.default.as_ref() {
+            arg = arg.default_value(default);
+        }
+        Ok(arg)
+    }
+
+    pub fn get_arg_value(&self, matches: &ArgMatches) -> Option<ArgcValue> {
         if !matches.contains_id(&self.name) {
             if let Some(default_fn) = self.default_fn.as_ref() {
                 return Some(ArgcValue::SingleFn(
@@ -228,33 +280,31 @@ impl Param for OptionParam {
             }
             return None;
         }
-        if self.multiple {
+        if self.multiple || self.arg_value_names.len() > 1 {
             let values = matches
                 .get_many::<String>(&self.name)
                 .map(|vals| vals.map(|v| escape_shell_words(v)).collect::<Vec<_>>())
                 .unwrap_or_default();
             Some(ArgcValue::Multiple(self.name.clone(), values))
         } else {
-            let value = escape_shell_words(matches.get_one::<String>(&self.name).unwrap());
+            let value = matches
+                .get_one::<String>(&self.name)
+                .map(|v| escape_shell_words(v))
+                .unwrap_or_default();
             Some(ArgcValue::Single(self.name.clone(), value))
         }
     }
 
-    fn detect_conflict(&self, names: &mut ParamNames, pos: Position) -> Result<()> {
+    pub fn detect_conflict(&self, names: &mut ParamNames, pos: Position) -> Result<()> {
         let tag_name = self.tag_name();
         detect_name_conflict(&self.name, false, tag_name, names, pos)?;
         detect_short_name_conflict(&self.short, tag_name, names, pos)
     }
-
-    fn is_positional(&self) -> bool {
-        false
-    }
 }
 
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Debug, PartialEq, Eq, Clone, Serialize)]
 pub struct PositionalParam {
     pub(crate) name: String,
-    pub(crate) value_name: Option<String>,
     pub(crate) summary: String,
     pub(crate) choices: Option<Vec<String>>,
     pub(crate) choices_fn: Option<String>,
@@ -262,6 +312,8 @@ pub struct PositionalParam {
     pub(crate) required: bool,
     pub(crate) default: Option<String>,
     pub(crate) default_fn: Option<String>,
+    pub(crate) value_name: Option<String>,
+    #[serde(skip_serializing)]
     pub(crate) arg_value_name: String,
 }
 
@@ -298,18 +350,16 @@ impl PositionalParam {
             arg_value_name: EXTRA_ARGS.to_string(),
         }
     }
-}
 
-impl Param for PositionalParam {
-    fn name(&self) -> &str {
+    pub fn name(&self) -> &str {
         &self.name
     }
 
-    fn tag_name(&self) -> &str {
+    pub fn tag_name(&self) -> &str {
         "@arg"
     }
 
-    fn render(&self) -> String {
+    pub fn render(&self) -> String {
         let mut output = vec![];
         let name = render_name(
             &self.name,
@@ -328,7 +378,7 @@ impl Param for PositionalParam {
         output.join(" ")
     }
 
-    fn build_arg(&self, index: usize) -> Result<Arg> {
+    pub fn build_arg(&self, index: usize) -> Result<Arg> {
         let mut arg = new_arg(&self.name, &self.summary);
         arg = arg
             .index(index + 1)
@@ -351,7 +401,7 @@ impl Param for PositionalParam {
         Ok(arg)
     }
 
-    fn get_arg_value(&self, matches: &ArgMatches) -> Option<ArgcValue> {
+    pub fn get_arg_value(&self, matches: &ArgMatches) -> Option<ArgcValue> {
         if !matches.contains_id(&self.name) {
             if let Some(default_fn) = self.default_fn.as_ref() {
                 return Some(ArgcValue::PositionalSingleFn(
@@ -373,20 +423,10 @@ impl Param for PositionalParam {
         }
     }
 
-    fn detect_conflict(&self, names: &mut ParamNames, pos: Position) -> Result<()> {
+    pub fn detect_conflict(&self, names: &mut ParamNames, pos: Position) -> Result<()> {
         let tag_name = self.tag_name();
         detect_name_conflict(&self.name, true, tag_name, names, pos)
     }
-
-    fn is_positional(&self) -> bool {
-        true
-    }
-}
-
-fn render_short(output: &mut Vec<String>, short: &Option<char>) {
-    if let Some(ch) = short {
-        output.push(format!("-{}", ch));
-    }
 }
 
 fn render_summary(output: &mut Vec<String>, summary: &str) {
@@ -406,8 +446,8 @@ fn render_name(
 ) -> String {
     let mut name = name.to_string();
     if let Some(choices) = choices {
-        if required {
-            name.push('!')
+        if let Some(ch) = get_modifer(required, multiple) {
+            name.push(ch)
         }
         let mut prefix = String::new();
         if default.is_some() {
@@ -426,8 +466,8 @@ fn render_name(
         let choices_value = format!("[{}{}]", prefix, values.join("|"));
         name.push_str(&choices_value);
     } else if let Some(choices_fn) = choices_fn {
-        if required {
-            name.push('!')
+        if let Some(ch) = get_modifer(required, multiple) {
+            name.push(ch)
         }
         let _ = write!(name, "[`{}`]", choices_fn);
     } else if let Some(default) = default {
@@ -439,15 +479,19 @@ fn render_name(
         let _ = write!(name, "={}", value);
     } else if let Some(default_fn) = default_fn {
         let _ = write!(name, "=`{}`", default_fn);
-    } else if let Some(ch) = match (required, multiple) {
+    } else if let Some(ch) = get_modifer(required, multiple) {
+        name.push(ch)
+    }
+    name
+}
+
+fn get_modifer(required: bool, multiple: bool) -> Option<char> {
+    match (required, multiple) {
         (true, true) => Some('+'),
         (true, false) => Some('!'),
         (false, true) => Some('*'),
         (false, false) => None,
-    } {
-        name.push(ch)
     }
-    name
 }
 
 fn new_arg(name: &str, summary: &str) -> Arg {
