@@ -6,11 +6,11 @@ use nom::{
     branch::alt,
     bytes::complete::{escaped, tag, take_till, take_while1},
     character::{
-        complete::{char, satisfy, space0, space1},
+        complete::{anychar, char, satisfy, space0, space1},
         streaming::none_of,
     },
     combinator::{eof, fail, map, opt, peek, rest, success},
-    multi::{many1, separated_list1},
+    multi::{many0, many1, separated_list1},
     sequence::{delimited, pair, preceded, separated_pair, terminated, tuple},
 };
 #[derive(Debug, PartialEq, Eq, Clone)]
@@ -29,8 +29,6 @@ pub enum EventData {
     Version(String),
     /// Author info
     Author(String),
-    /// Help subcommand
-    Help(String),
     /// Define a subcommand, e.g. `@cmd A sub command`
     Cmd(String),
     /// Define alias for a subcommand, e.g. `@alias t,tst`
@@ -121,13 +119,7 @@ fn parse_tag(input: &str) -> nom::IResult<&str, Option<EventData>> {
 fn parse_tag_text(input: &str) -> nom::IResult<&str, Option<EventData>> {
     map(
         pair(
-            alt((
-                tag("describe"),
-                tag("version"),
-                tag("author"),
-                tag("cmd"),
-                tag("help"),
-            )),
+            alt((tag("describe"), tag("version"), tag("author"), tag("cmd"))),
             parse_tail,
         ),
         |(tag, text)| {
@@ -135,7 +127,6 @@ fn parse_tag_text(input: &str) -> nom::IResult<&str, Option<EventData>> {
             Some(match tag {
                 "describe" => EventData::Describe(text),
                 "version" => EventData::Version(text),
-                "help" => EventData::Help(text),
                 "author" => EventData::Author(text),
                 "cmd" => EventData::Cmd(text),
                 _ => unreachable!(),
@@ -181,26 +172,59 @@ fn parse_tag_unknown(input: &str) -> nom::IResult<&str, Option<EventData>> {
 
 // Parse `@option`
 fn parse_option_param(input: &str) -> nom::IResult<&str, OptionParam> {
+    alt((parse_with_long_option_param, parse_no_long_option_param))(input)
+}
+
+// Parse `@option` with long name
+fn parse_with_long_option_param(input: &str) -> nom::IResult<&str, OptionParam> {
     map(
         tuple((
             parse_short,
             preceded(
                 pair(space0, tag("--")),
                 alt((
-                    parse_param_choices_fn_required,
-                    parse_param_choices_fn,
-                    parse_param_choices_default,
-                    parse_param_choices_required,
-                    parse_param_choices,
+                    parse_param_modifer_choices_default,
+                    parse_param_modifer_choices_fn,
+                    parse_param_modifer_choices,
                     parse_param_assign_fn,
                     parse_param_assign,
-                    parse_param_mark,
+                    parse_param_modifer,
                 )),
             ),
-            parse_value_notation,
+            parse_zero_or_many_value_notations,
             parse_tail,
         )),
-        |(short, arg, value_name, summary)| OptionParam::new(arg, summary, short, value_name),
+        |(short, arg, value_names, summary)| {
+            OptionParam::new(arg, summary, short, false, &value_names)
+        },
+    )(input)
+}
+
+// Parse `@option` without long name
+fn parse_no_long_option_param(input: &str) -> nom::IResult<&str, OptionParam> {
+    map(
+        tuple((
+            preceded(
+                pair(space0, tag("-")),
+                preceded(
+                    verify_single_char,
+                    alt((
+                        parse_param_modifer_choices_default,
+                        parse_param_modifer_choices_fn,
+                        parse_param_modifer_choices,
+                        parse_param_assign_fn,
+                        parse_param_assign,
+                        parse_param_modifer,
+                    )),
+                ),
+            ),
+            parse_zero_or_many_value_notations,
+            parse_tail,
+        )),
+        |(arg, value_names, summary)| {
+            let short = arg.name.chars().next();
+            OptionParam::new(arg, summary, short, true, &value_names)
+        },
     )(input)
 }
 
@@ -209,16 +233,14 @@ fn parse_positional_param(input: &str) -> nom::IResult<&str, PositionalParam> {
     map(
         tuple((
             alt((
-                parse_param_choices_fn_required,
-                parse_param_choices_fn,
-                parse_param_choices_default,
-                parse_param_choices_required,
-                parse_param_choices,
+                parse_param_modifer_choices_default,
+                parse_param_modifer_choices_fn,
+                parse_param_modifer_choices,
                 parse_param_assign_fn,
                 parse_param_assign,
-                parse_param_mark,
+                parse_param_modifer,
             )),
-            parse_value_notation,
+            parse_zero_or_one_value_notation,
             parse_tail,
         )),
         |(arg, value_name, summary)| PositionalParam::new(arg, summary, value_name),
@@ -227,18 +249,36 @@ fn parse_positional_param(input: &str) -> nom::IResult<&str, PositionalParam> {
 
 // Parse `@flag`
 fn parse_flag_param(input: &str) -> nom::IResult<&str, FlagParam> {
+    alt((parse_with_long_flag_param, parse_no_long_flag_param))(input)
+}
+// Parse `@flag`
+fn parse_with_long_flag_param(input: &str) -> nom::IResult<&str, FlagParam> {
     map(
         tuple((
             parse_short,
-            preceded(pair(space0, tag("--")), parse_param_multiple),
+            preceded(pair(space0, tag("--")), parse_long_flag_and_asterisk),
             parse_tail,
         )),
-        |(short, arg, summary)| FlagParam::new(arg, summary, short),
+        |(short, arg, summary)| FlagParam::new(arg, summary, short, false),
+    )(input)
+}
+
+// Parse `@flag` without long name
+fn parse_no_long_flag_param(input: &str) -> nom::IResult<&str, FlagParam> {
+    map(
+        tuple((
+            preceded(pair(space0, tag("-")), parse_short_flag_and_asterisk),
+            parse_tail,
+        )),
+        |(arg, summary)| {
+            let short = arg.name.chars().next();
+            FlagParam::new(arg, summary, short, true)
+        },
     )(input)
 }
 
 // Parse `str*` `str`
-fn parse_param_multiple(input: &str) -> nom::IResult<&str, ParamData> {
+fn parse_long_flag_and_asterisk(input: &str) -> nom::IResult<&str, ParamData> {
     alt((
         map(terminated(parse_param_name, tag("*")), |mut arg| {
             arg.multiple = true;
@@ -248,8 +288,21 @@ fn parse_param_multiple(input: &str) -> nom::IResult<&str, ParamData> {
     ))(input)
 }
 
+// Parse ':' or '#' or '0'
+fn parse_short_flag_and_asterisk(input: &str) -> nom::IResult<&str, ParamData> {
+    fn parser(input: &str) -> nom::IResult<&str, ParamData> {
+        map(satisfy(is_short_char), |ch| {
+            ParamData::new(&format!("{}", ch))
+        })(input)
+    }
+    map(pair(parser, opt(tag("*"))), |(mut arg, multiple)| {
+        arg.multiple = multiple.is_some();
+        arg
+    })(input)
+}
+
 // Parse `str!` `str*` `str+` `str`
-fn parse_param_mark(input: &str) -> nom::IResult<&str, ParamData> {
+fn parse_param_modifer(input: &str) -> nom::IResult<&str, ParamData> {
     alt((
         map(terminated(parse_param_name, tag("!")), |mut arg| {
             arg.required = true;
@@ -290,74 +343,42 @@ fn parse_param_assign_fn(input: &str) -> nom::IResult<&str, ParamData> {
     )(input)
 }
 
-// Parse `str[a|b|c]`
-fn parse_param_choices(input: &str) -> nom::IResult<&str, ParamData> {
+fn parse_param_modifer_choices_default(input: &str) -> nom::IResult<&str, ParamData> {
     map(
         pair(
-            parse_param_name,
-            delimited(char('['), parse_choices, char(']')),
-        ),
-        |(mut arg, choices)| {
-            arg.choices = Some(choices.iter().map(|v| v.to_string()).collect());
-            arg
-        },
-    )(input)
-}
-
-// Parse str[`fn`]
-fn parse_param_choices_fn(input: &str) -> nom::IResult<&str, ParamData> {
-    map(
-        pair(
-            parse_param_name,
-            delimited(char('['), parse_value_fn, char(']')),
-        ),
-        |(mut arg, choices_fn)| {
-            arg.choices_fn = Some(choices_fn.into());
-            arg
-        },
-    )(input)
-}
-
-// Parse `str[=a|b|c]`
-fn parse_param_choices_default(input: &str) -> nom::IResult<&str, ParamData> {
-    map(
-        pair(
-            parse_param_name,
+            parse_param_modifer,
             delimited(char('['), parse_choices_default, char(']')),
         ),
         |(mut arg, (choices, default))| {
             arg.choices = Some(choices.iter().map(|v| v.to_string()).collect());
+            arg.required = false;
             arg.default = default.map(|v| v.to_string());
             arg
         },
     )(input)
 }
 
-// Parse `str![a|b|c]`
-fn parse_param_choices_required(input: &str) -> nom::IResult<&str, ParamData> {
+fn parse_param_modifer_choices(input: &str) -> nom::IResult<&str, ParamData> {
     map(
         pair(
-            terminated(parse_param_name, char('!')),
+            parse_param_modifer,
             delimited(char('['), parse_choices, char(']')),
         ),
         |(mut arg, choices)| {
             arg.choices = Some(choices.iter().map(|v| v.to_string()).collect());
-            arg.required = true;
             arg
         },
     )(input)
 }
 
-// Parse str![`fn`]
-fn parse_param_choices_fn_required(input: &str) -> nom::IResult<&str, ParamData> {
+fn parse_param_modifer_choices_fn(input: &str) -> nom::IResult<&str, ParamData> {
     map(
         pair(
-            terminated(parse_param_name, char('!')),
+            parse_param_modifer,
             delimited(char('['), parse_value_fn, char(']')),
         ),
         |(mut arg, choices_fn)| {
             arg.choices_fn = Some(choices_fn.into());
-            arg.required = true;
             arg
         },
     )(input)
@@ -370,22 +391,23 @@ fn parse_param_name(input: &str) -> nom::IResult<&str, ParamData> {
 
 // Parse `-s`
 fn parse_short(input: &str) -> nom::IResult<&str, Option<char>> {
-    let short = delimited(
-        char('-'),
-        satisfy(|c| c.is_ascii_alphabetic()),
-        peek(space1),
-    );
+    let short = delimited(char('-'), satisfy(is_short_char), peek(space1));
     opt(short)(input)
 }
 
+// Zero or many '<FOO>'
+fn parse_zero_or_many_value_notations(input: &str) -> nom::IResult<&str, Vec<&str>> {
+    many0(parse_value_notation)(input)
+}
+
+// Zero or one '<FOO>'
+fn parse_zero_or_one_value_notation(input: &str) -> nom::IResult<&str, Option<&str>> {
+    opt(parse_value_notation)(input)
+}
+
 // Parse '<FOO>'
-fn parse_value_notation(input: &str) -> nom::IResult<&str, Option<&str>> {
-    let main = delimited(
-        char('<'),
-        take_while1(|c: char| !(c.is_whitespace() || c == '>')),
-        char('>'),
-    );
-    opt(preceded(space0, main))(input)
+fn parse_value_notation(input: &str) -> nom::IResult<&str, &str> {
+    preceded(space0, delimited(char('<'), parse_notation_text, char('>')))(input)
 }
 
 // Parse `a|b|c`
@@ -459,6 +481,45 @@ fn parse_quoted_string(input: &str) -> nom::IResult<&str, &str> {
     alt((single, double))(input)
 }
 
+fn parse_notation_text(input: &str) -> nom::IResult<&str, &str> {
+    let (_, size) = notation_text(input, 1)?;
+    Ok((&input[size - 1..], &input[0..size - 1]))
+}
+
+fn notation_text(input: &str, balances: usize) -> nom::IResult<&str, usize> {
+    let (i1, c1) = anychar(input)?;
+    match c1 {
+        '<' => {
+            let (i2, count) = notation_text(i1, balances + 1)?;
+            Ok((i2, count + 1))
+        }
+        '>' => {
+            if balances == 1 {
+                Ok((i1, 1))
+            } else {
+                let (i2, count) = notation_text(i1, balances - 1)?;
+                Ok((i2, count + 1))
+            }
+        }
+        _ => notation_text(i1, balances).map(|(i3, v)| (i3, 1 + v)),
+    }
+}
+
+fn verify_single_char(input: &str) -> nom::IResult<&str, &str> {
+    if input
+        .chars()
+        .take_while(|v| v.is_ascii_alphanumeric())
+        .count()
+        > 1
+    {
+        return Err(nom::Err::Error(nom::error::Error::new(
+            input,
+            nom::error::ErrorKind::Verify,
+        )));
+    }
+    Ok((input, ""))
+}
+
 fn is_not_fn_name_char(c: char) -> bool {
     !matches!(
         c,
@@ -483,13 +544,16 @@ fn is_not_fn_name_char(c: char) -> bool {
 }
 
 fn is_name_char(c: char) -> bool {
-    c.is_ascii_alphanumeric() || c == '_' || c == '-'
+    c.is_ascii_alphanumeric() || matches!(c, '_' | '-' | '.')
+}
+
+fn is_short_char(c: char) -> bool {
+    c.is_ascii() && is_not_fn_name_char(c) && !matches!(c, '-')
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::param::Param;
 
     macro_rules! assert_token {
         ($comment:literal, Ignore) => {
@@ -524,7 +588,7 @@ mod tests {
     }
 
     macro_rules! assert_parse_option_arg {
-        ($data:literal, &expect:literal) => {
+        ($data:literal, $expect:literal) => {
             assert_eq!(
                 parse_option_param($data).unwrap().1.render().as_str(),
                 $expect
@@ -539,7 +603,7 @@ mod tests {
     }
 
     macro_rules! assert_parse_flag_arg {
-        ($data:literal, &expect:literal) => {
+        ($data:literal, $expect:literal) => {
             assert_eq!(parse_flag_arg($data).unwrap().1.render().as_str(), $expect);
         };
         ($data:literal) => {
@@ -548,7 +612,7 @@ mod tests {
     }
 
     macro_rules! assert_parse_positional_arg {
-        ($data:literal, &expect:literal) => {
+        ($data:literal, $expect:literal) => {
             assert_eq!(
                 parse_positional_param($data).unwrap().1.render().as_str(),
                 $expect
@@ -563,7 +627,7 @@ mod tests {
     }
 
     #[test]
-    fn test_parse_option_arg() {
+    fn test_parse_with_long_option_arg() {
         assert_parse_option_arg!("-f --foo=a <FOO> A foo option");
         assert_parse_option_arg!("--foo!");
         assert_parse_option_arg!("--foo+");
@@ -574,24 +638,65 @@ mod tests {
         assert_parse_option_arg!("--foo[a|b]");
         assert_parse_option_arg!("--foo[=a|b]");
         assert_parse_option_arg!("--foo[`_foo`]");
+        assert_parse_option_arg!("--foo![a|b]");
+        assert_parse_option_arg!("--foo![`_foo`]");
+        assert_parse_option_arg!("--foo![=a|b]", "--foo[=a|b]");
+        assert_parse_option_arg!("--foo+[a|b]");
+        assert_parse_option_arg!("--foo+[`_foo`]");
+        assert_parse_option_arg!("--foo+[=a|b]", "--foo*[=a|b]");
+        assert_parse_option_arg!("--foo*[a|b]");
+        assert_parse_option_arg!("--foo*[=a|b]");
+        assert_parse_option_arg!("--foo*[`_foo`]");
         assert_parse_option_arg!("--foo <FOO>");
         assert_parse_option_arg!("--foo-abc <FOO>");
         assert_parse_option_arg!("--foo=\"a b\"");
         assert_parse_option_arg!("--foo[\"a|b\"|\"c]d\"]");
-        assert_parse_option_arg!("--foo![a|b]");
-        assert_parse_option_arg!("--foo![`_foo`]");
+        assert_parse_option_arg!("--foo <abc>");
+        assert_parse_option_arg!("--foo <abc> <def>");
+        assert_parse_option_arg!("--foo <>");
+        assert_parse_option_arg!("--foo <abc def>");
+        assert_parse_option_arg!("--foo <<abc def>>");
     }
 
     #[test]
-    fn test_parse_flag_arg() {
+    fn test_parse_no_long_option_arg() {
+        assert_parse_option_arg!("-f");
+        assert_parse_option_arg!("-f!");
+        assert_parse_option_arg!("-f=a");
+        assert_parse_option_arg!("-f=`_foo`");
+        assert_parse_option_arg!("-f[a|b]");
+        assert_parse_option_arg!("-f[=a|b]");
+        assert_parse_option_arg!("-f[`_foo`]");
+        assert_parse_option_arg!("-f![a|b]");
+        assert_parse_option_arg!("-f![`_foo`]");
+        assert_parse_option_arg!("-f![=a|b]", "-f[=a|b]");
+    }
+
+    #[test]
+    fn test_parse_with_long_flag_arg() {
         assert_parse_flag_arg!("-f --foo A foo flag");
+        assert_parse_flag_arg!("-. --hidden");
+        assert_parse_flag_arg!("--http1.1");
         assert_parse_flag_arg!("--foo A foo flag");
         assert_parse_flag_arg!("--foo");
+        assert_parse_flag_arg!("--foo*");
+    }
+
+    #[test]
+    fn test_parse_no_long_flag_arg() {
+        assert_parse_flag_arg!("-f A foo flag");
+        assert_parse_flag_arg!("-f");
+        assert_parse_flag_arg!("-.");
+        assert_parse_flag_arg!("-0");
+        assert_parse_flag_arg!("-#");
+        assert_parse_flag_arg!("-:");
+        assert_parse_flag_arg!("-f*");
     }
 
     #[test]
     fn test_parse_positional_arg() {
         assert_parse_positional_arg!("foo <FOO> A foo arg");
+        assert_parse_positional_arg!("a.b");
         assert_parse_positional_arg!("foo");
         assert_parse_positional_arg!("foo!");
         assert_parse_positional_arg!("foo+");
@@ -604,6 +709,13 @@ mod tests {
         assert_parse_positional_arg!("foo[=a|b]");
         assert_parse_positional_arg!("foo![a|b]");
         assert_parse_positional_arg!("foo![`_foo`]");
+        assert_parse_positional_arg!("foo![=a|b]", "foo[=a|b]");
+        assert_parse_positional_arg!("foo+[a|b]");
+        assert_parse_positional_arg!("foo+[`_foo`]");
+        assert_parse_positional_arg!("foo+[=a|b]", "foo*[=a|b]");
+        assert_parse_positional_arg!("foo*[a|b]");
+        assert_parse_positional_arg!("foo*[`_foo`]");
+        assert_parse_positional_arg!("foo*[=a|b]");
     }
 
     #[test]
@@ -637,8 +749,7 @@ mod tests {
         assert_token!("function foo@bar", Func, "foo@bar");
         assert_token!("foo=bar", Ignore);
         assert_token!("#!/bin/bash", Ignore);
-        assert_token!("# @flag -f", Error);
+        assert_token!("# @flag -foo", Error);
         assert_token!("# @option -foo![=a|b]", Error);
-        assert_token!("# @arg foo![=a|b]", Error);
     }
 }
