@@ -17,6 +17,22 @@ cmd_omitted() {
     print_argc_vars
 }
 
+# @cmd Flags all kind of names
+# @flag --flag1
+# @flag --flag2       This is a flag
+# @flag --flag3*
+# @flag -f --flag4    This is a flag
+cmd_flag_names() {
+    print_argc_vars
+}
+
+# @cmd Flags without long name
+# @flag -a    This is a flag
+# @flag -b*
+cmd_no_long_flags() {
+    print_argc_vars
+}
+
 # @cmd Options all kind of names
 # @option      --opt1                optional
 # @option      --opt2!               required
@@ -33,6 +49,23 @@ cmd_option_names() {
     print_argc_vars
 }
 
+# @cmd Options without long name
+# @option      -a                optional
+# @option      -b!               required
+# @option      -c*               optional, multiple
+# @option      -d+               required, multiple
+# @option      -e=a              optional, default
+# @option      -f[a|b|c]         choices
+# @option      -g[=a|b|c]        choices, default
+# @option      -i![a|b|c]        required, choices
+# @option      -j[`_fn_foo`]     optional, default from fn
+# @option      -k[`_fn_bars`]    choices from fn
+# @option      -l![`_fn_bars`]   required, choices from fn
+# @option      -m <VALUE>        optional, default
+cmd_no_long_options() {
+    print_argc_vars
+}
+
 # @cmd Options all kind of formats
 # @option      --opt1
 # @option  -a  --opt2
@@ -55,6 +88,13 @@ cmd_option_quotes() {
     print_argc_vars
 }
 
+# @cmd Option with value notations
+# @option   --opt1 <DIR>
+# @option   --opt2 <DIR> <FILE>
+cmd_option_notations() {
+    print_argc_vars
+}
+
 # @cmd All kind of flags
 # @flag      --foo1
 # @flag   -a --foo2    
@@ -66,14 +106,21 @@ cmd_flag_formats() {
 }
 
 # @cmd  Positional one required
-# @arg   arg1! <ARG>  A required arg
+# @arg   arg1! <DIR>  A required arg1
 cmd_positional_only() {
     print_argc_vars
 }
 
 # @cmd  Positional all required
-# @arg   arg1!     A required arg
-# @arg   arg2+     A required arg, multiple
+# @arg   dir1     A optional arg 1
+# @arg   dir2     A optional arg 2
+cmd_positional_many() {
+    print_argc_vars
+}
+
+# @cmd  Positional all required
+# @arg   dir1!     A required arg
+# @arg   file2+     A required arg 2, multiple
 cmd_positional_requires() {
     print_argc_vars
 }
@@ -147,6 +194,7 @@ cmd_nested_command() {
 }
 
 # @cmd Subcommand of nested command
+# @alias a
 # @option --opt1
 # @option --opt2
 cmd_nested_command::foo() {
@@ -167,21 +215,59 @@ cmd_nested_command2::foo() {
     print_argc_vars
 }
 
-
 cmd_nested_command2::main() {
     print_argc_vars
 }
 
+# @cmd Notation values
+# @option --opt1 <>
+# @option --opt2 <ABC>
+# @option --opt3 <ABC DEF>
+# @option --opt4 <[ABC DEF]>
+# @option --opt5 <<ABC DEF>>
+cmd_notation_values() {
+    print_argc_vars
+}
+
+# @cmd Dynamic compgen
+# @arg args*[`_fn_args`]
+cmd_dynamic_compgen() {
+    print_argc_vars
+}
+
 print_argc_vars() {
     ( set -o posix ; set ) | grep argc_
 }
 
 _fn_foo() {
-    echo "foo"
+    echo foo
 }
 
 _fn_bars() {
-    echo " a1 a2 a3 "
+    echo a1
+    echo a2
+    echo a3
 }
 
-eval "$(argc "$0" "$@")"
+_fn_args() {
+    echo $@
+    ( set -o posix ; set ) | grep argc_
+}
+
+_fn_dup() {
+    case $(basename ${SHELL}) in
+        zsh)
+            _local_func() {
+                echo "do this"
+            }
+            ;;
+
+        bash)
+            _local_func() {
+                echo "do that instead"
+            }
+            ;;
+    esac
+}
+
+eval "$(argc --argc-eval "$0" "$@")"
